<?php
/*
Plugin Name: FriendFeed Comments
Version: 1.5.2
Plugin URI: http://blog.slaven.net.au/wordpress-plugins/friendfeed-comments-wordpress-plugin/
Description: Places the comments & likes from FriendFeed on each post from your blog under the normal comments section on the post page.
Author: Glenn Slaven
Author URI: http://blog.slaven.net.au/

	Copyright 2008 Glenn Slaven  (email : gdalziel@gmail.com)
*/
if (!class_exists('FriendFeed')) {
	require_once("friendfeed.php");
}

class FriendFeedPostComments
{
	var $options_key = 'friendfeed_options';
	var $apikeys_key = 'friendfeed_apikeys';
	var $discussions_key = 'friendfeed_discussions';
	var $cookie_key = 'wp_ffcomments';
	var $max_posts_to_load = 150;
	var $page_size = 30;
	var $plugin_name = "FriendFeed Comments";
	var $is_debug = false;
	var $plugin_path;
	
	function FriendFeedPostComments()
	{
		add_action('admin_menu', array(&$this, '_create_options_menu_item'));
		add_action('update_friendfeedcomments_hook', array(&$this, 'update_friendfeedcomments'));
		add_action('wp_head', array(&$this, 'add_page_header'));
		
		register_activation_hook(__FILE__, array(&$this, '_install'));
		register_deactivation_hook(__FILE__, array(&$this, '_uninstall'));
		
		$this->plugin_path = get_option('siteurl') . '/' . PLUGINDIR . '/friendfeed-comments/';
		
		wp_enqueue_script('prototype');
	}
	
	//Callback to create the Settings menu option
	function _create_options_menu_item(){
		add_options_page($this->plugin_name, $this->plugin_name, 9, __FILE__, array(&$this, 'options_page'));
		add_action('admin_notices', array(&$this, '_admin_notices'));
	}
	
	//Display admin notices if needed
	function _admin_notices() {
		$options = get_option($this->options_key);
		if ((!$options || !$options['nickname']) && !$_POST['ff_nickname'])
		{
			echo "<div class='updated' style='background-color:#f66;'><p><a href=\"options-general.php?page=friendfeed-comments/friendfeed-comments.php\">FriendFeed Comments</a> does not have a nickname set.  You need to enter your FriendFeed nickname for the comments to be loaded.</p></div>";
		}
	}
	
	//Check if the debug flag is set
	function is_debug() {
		return $this->is_debug || $_REQUEST['ffdebugflag'];
	}
	
	//Callback to setup the plugin
	function _install() {
		wp_schedule_event( time(), 'hourly', 'update_friendfeedcomments_hook' );
		$options = get_option($this->options_key);
		if (!$options) {
				$options['nickname'] = '';
				$options['servicename'] = 'blog';
				$options['showcomments'] = false;
				$options['headline'] = false;
				$options['showform'] = true;
				$options['usestyles'] = true;
			update_option($this->options_key, $options);
		}
		
	}
	
	//Callback to unstall the plugin
	function _uninstall() {
		wp_clear_scheduled_hook('update_friendfeedcomments_hook');
	}
	
	//Delete the stored username & password for the user & remove the cookie
	function remove_authentication() {
		if ($_COOKIE[$this->cookie_key]) {
			$hash = $_COOKIE[$this->cookie_key];
			
			$apikeys = get_option($this->apikeys_key);
			if (is_array($apikeys)) {
				unset($apikeys[$hash]);
			}
			
			update_option($this->apikeys_key, $apikeys);
			
			setcookie($this->cookie_key, '', time() - 31536000, COOKIEPATH, COOKIE_DOMAIN);
		}
	}
	
	//Update the apikey for the user
	function update_apikey($username, $apikey) {
		$apikeys = get_option($this->apikeys_key);
		if (!is_array($apikeys)) {
			$apikeys = array();
		}
		
		$hash = md5($username + time());
		
		$apikeys[$hash] = array($username, $apikey);
		
		update_option($this->apikeys_key, $apikeys);
		setcookie($this->cookie_key, $hash, time() + 31536000, COOKIEPATH, COOKIE_DOMAIN);
	}
	
	//Get the username & apikey stored for a user given the cookie
	function get_authentication() {
		$auth = false;
		
		if ($_COOKIE[$this->cookie_key]) {
			$hash = $_COOKIE[$this->cookie_key];	
			$apikeys = get_option($this->apikeys_key);
			
			if (is_array($apikeys)) {
				$auth = $apikeys[$hash];
			}
		}
		
		return $auth;
	}
	
	//Format the title into a normalised string that can be used as a hash key
	function format_title($title) {;
		$normalised = $title;
		$normalised = str_replace(array("", "", "", "", "`", '"', "'", ',', ':', urldecode('%93'), urldecode('&#8217;'), urldecode('&#8216;'), urldecode('&#8220;'), urldecode('&#8221;')), '', $normalised);
		$normalised = urlencode($normalised);
		$normalised = preg_replace("/%E2%80%9./i", '', $normalised);

		if ($this->is_debug()) {
			$this->printr($title . ' - ' . $normalised);
		}
		
		return md5($normalised);
	}
	
	//Debug output
	function printr($string) {
		print "<pre>";
		print_r($string);
		print "</pre>";
	}
	
	//Callback to update the comments from the FriendFeed API
	function update_friendfeedcomments() {
		$options = get_option($this->options_key);
		$friendfeed = new FriendFeed($options['nickname']);
		$ffposts = $friendfeed->fetch_user_feed($options['nickname'], $options['servicename']);
		
		if ($ffposts)
		{
			//Load up the posts from FriendFeed up to the max number of posts to load set
			for ($i = $this->page_size ; $i <= $this->max_posts_to_load - $this->page_size; $i = $i + $this->page_size) {
				$temp_posts = $friendfeed->fetch_user_feed($options['nickname'], $options['servicename'], $i, $this->page_size);
				
				if ($temp_posts && is_array($temp_posts->entries)) {
					$ffposts->entries = array_merge($ffposts->entries, $temp_posts->entries);
				} else {
					$i = $this->max_posts_to_load; 
				}
			}
			
			if ($this->is_debug()) {
				$this->printr(count($ffposts->entries) . " entries loaded from FriendFeed");
			}
		
			$ffdiscussions = get_option($this->discussions_key);
			$blog_url = parse_url(get_option('siteurl'));
			
			$match_count = 0;
			foreach($ffposts->entries as $post)
			{
				$post_url = parse_url($post->service->profileUrl);
				$is_same_blog = ($blog_url['host'] == $post_url['host']) && (
					$blog_url['path'] == '/' || 
					$post_url['path'] == '/' || 
					$post_url['path'] == $blog_url['path']
				); 
				if ($is_same_blog)
				{
					$title = $this->format_title($post->title);
					$ffdiscussions[$title]['id'] = $post->id;
					$ffdiscussions[$title]['title'] = $post->title;
					$ffdiscussions[$title]['link'] = $post->link;
					$ffdiscussions[$title]['comments'] = $post->comments;
					$ffdiscussions[$title]['likes'] = $post->likes;
					$match_count++;
				}
			}	
			update_option($this->discussions_key, $ffdiscussions);
		
			if ($this->is_debug()) {
				$this->printr("$match_count entries loaded, " . count($ffdiscussions) . " total entries stored.");
				$this->printr($ffdiscussions);
				$this->printr($ffposts);
			}

		}	
	}
	
	//Add the needed items to the page header
	function add_page_header() {
		$options = get_option($this->options_key);
		
		if ($options['usestyles']) {
			print "\t<link rel=\"stylesheet\" type=\"text/css\" href=\"" . $this->plugin_path ."defaultStyle.css\" />\n";
		}
		
?>
	<script type="text/javascript">var friendFeedServicePath = '<?=$this->plugin_path?>';</script>
	<script  type="text/javascript" src="<?=$this->plugin_path?>friendfeed-comments.js"></script>
<?php
	}
	
	//Function to render the options page
	function options_page() {
		$options = get_option($this->options_key);
		if ($_POST["wp_friendfeed"])
		{
			if ($_POST['ff_showdebug']) {
				$this->is_debug = true;
			}
		
			if ($_POST['Clear']) {
				delete_option($this->discussions_key);
				$this->update_friendfeedcomments();
				$message = "<strong>Discussions reloaded.</strong>";
			} elseif ($_POST['Delete']) { 
				delete_option($this->discussions_key);
				delete_option($this->options_key);
				$_POST['ff_nickname'] = '';
				$_POST['ff_servicename'] = '';
				$_POST['ff_showcomments'] = '';
				$_POST['ff_headline'] = '';
				$_POST['ff_showform'] = '';				
				$_POST['ff_usestyles'] = '';				
				$message = "<strong>All information related to this plugin has been deleted.</strong> To finish removing this plugin, deactivate it on the Plugins page";
			} else {
				$options['nickname'] = $_POST['ff_nickname'];
				$options['servicename'] = $_POST['ff_servicename'];
				$options['showcomments'] = $_POST['ff_showcomments'];
				$options['headline'] = $_POST['ff_headline'];
				$options['showform'] = $_POST['ff_showform'];
				$options['usestyles'] = $_POST['ff_usestyles'];
				update_option($this->options_key, $options);
				$this->update_friendfeedcomments();
				$message = "<strong>Settings Saved.</strong>  Don't forget to put <code>&lt;?php wp_ffcomments(); ?&gt;</code> on your single post template to display the FriendFeed comments &amp; likes.";
			}
?>
	<div id="message" class="updated fade" style="background-color: rgb(255, 251, 204);"><p><?=$message?></p></div>
<?php
		}
?>
	<div class="wrap">
		<form method="post">
			<input type="hidden" value="true" name="wp_friendfeed"/>
			<h2>FriendFeed Comments</h2>
			<table class="form-table">
			<tbody>
				<tr valign="top">
					<th scope="row">FriendFeed Nickname</th>
					<td><input type="text" size="40" value="<?=$options['nickname']?>" id="ff_nickname" name="ff_nickname" /></td>
				</tr>
				<tr valign="top">
					<th scope="row">Use default stylesheet</th>
					<td>
						<input type="checkbox" name="ff_usestyles" id="ff_usestyles" value="1"<?=($options['usestyles'] ? ' checked="checked"' : '')?> />
						<br />If this is checked the plugin will import the default stylesheet for styling the display of the comments &amp; likes
						lists.  This is meant as a starting point, please feel free to use the styles &amp; adapt them for your site.<br />
						<a href="<?=$this->plugin_path?>defaultStyle.css" target="_BLANK">Click here to view the default stylesheet</a>.
					</td>
				</tr>		
				<tr valign="top">
					<th scope="row">Enable comments &amp; likes form</th>
					<td>
						<input type="checkbox" name="ff_showform" id="ff_showform" value="1"<?=($options['showform'] ? ' checked="checked"' : '')?> />
						<br />If this is checked the plugin will display a small form under the comments to allow the reader to add a comment or 
						'like' on the post FriendFeed.  
					</td>
				</tr>					
				<tr valign="top">
					<th scope="row">Headline Text</th>
					<td>
						<input type="text" size="100" value="<?=$options['headline']?>" id="ff_headline" name="ff_headline" />
						<br />This is the text that will be shown above the comments & likes. The tokens {comments} &amp; {likes} will
						be replaced with the number of comments &amp; likes for the post.  Leave this blank to use the default 
						"<em>On FriendFeed, this post was liked by x people and commented on x times</em>".
					</td>
				</tr>					
				<tr valign="top">
					<th scope="row">Show comments by default</th>
					<td>
						<input type="checkbox" name="ff_showcomments" id="ff_showcomments" value="1"<?=($options['showcomments'] ? ' checked="checked"' : '')?> />
						<br />If this is checked the FriendFeed comments &amp; likes will be shown by default.  If not checked the user will have to click on the 'show'
						link to display them.
					</td>
				</tr>	
				<tr valign="top">
					<th scope="row">FriendFeed Servicename</th>
					<td>
						<input type="text" size="40" value="<?=$options['servicename']?>" id="ff_servicename" name="ff_servicename" />
						<br />This is the name that FriendFeed knows your blog as.  Unless you don't use the 'blog' service on FriendFeed, you probably won't need to change this.
					</td>
				</tr>
				<tr valign="top">
					<th scope="row">Display debug information</th>
					<td>
						<input type="checkbox" name="ff_showdebug" id="ff_showdebug" value="1" />
						<br />Check this if you want to see the information that is being loaded by the plugin when you save.  Only needed for troubleshooting.
					</td>
				</tr>
			</tbody>
			</table>
			<p class="submit">
				<input type="submit" value="Save Changes" name="Submit"/>
				<input type="submit" style="color:#DD0000; font-weight:bold;" value="Reset comments/likes" name="Clear" onclick="return confirm('This will delete then reload all FriendFeed comments & likes.  Are you sure?');" />
				<input type="submit" style="color:#DD0000; font-weight:bold;" value="Delete plugin" name="Delete" onclick="return confirm('This will delete all information relating to this plugin.  Only do this if you're intending to remove this plugin.  Are you sure?');" />
			</p>
		</form>
		<div style="background-color:rgb(238, 238, 238); border: 1px solid rgb(85, 85, 85); padding: 5px; margin-top:10px;">
			<p>Did you find this plugin useful?  Please consider donating to help me continue developing it and other plugins.</p>
			<form action="https://www.paypal.com/cgi-bin/webscr" method="post"><input type="hidden" name="cmd" value="_xclick" /><input type="hidden" name="business" value="paypal@slaven.net.au" /><input type="hidden" name="item_name" value="FriendFeed Comments Wordpress Plugin" /><input type="hidden" name="no_note" value="1" /><input type="hidden" name="currency_code" value="AUD" /><input type="hidden" name="tax" value="0" /><input type="hidden" name="bn" value="PP-DonationsBF" /><input type="image" src="https://www.paypal.com/en_US/i/btn/x-click-but04.gif" border="0" name="submit" alt="Make payments with PayPal - it's fast, free and secure!" /></form>
		</div>
	</div>
<?php
	}
	
	//Render the list of likes passed in
	function render_likes_list($likes) {
		$output_string = '';
		
		$count = count($likes);
		if ($count) {
			$output_string .= "Liked by\n\t\t\t<ul>\n";
			for($i = 0; $i < $count; $i++) {
				$like = $likes[$i];
				$output_string .= "\t\t\t\t<li><a href=\"{$like->user->profileUrl}\">{$like->user->name}</a>" . ($i < $count - 1 ? ", " : '') . "</li>\n";
			}
			$output_string .= "\t\t\t</ul>\n";
		}
		return $output_string;
	}
	
	//Build up the string to display the list of comments
	function render_comments_list($comments) {
		$output_string = '';
		
		$output_string .= "<ul>\n";
		foreach($comments as $comment) {
			$output_string .= $this->render_comment($comment);
		}
		$output_string .= "</ul>\n";
		
		return $output_string;
	}
	
	//Render an individual comment
	function render_comment($comment) {
		$output_string = '';
		$timeformat = get_option('time_format');
		$dateformat = get_option('date_format');		
		$time = date($timeformat, $comment->date);
		$date = date($dateformat, $comment->date);
		$output_string .= "\t\t\t<li class=\"friendfeedcomment\">\n";
		$output_string .= "\t\t\t\t<a href=\"{$comment->user->profileUrl}\"><img style=\"float:left; margin: 0 5px 5px 0px;\" src=\"http://friendfeed.com/{$comment->user->nickname}/picture?size=medium\" alt=\"\" /></a>";
		$output_string .= "\t\t\t\t<div class=\"friendfeedmeta\"><em>{$date} at {$time}</em> <cite><a href=\"{$comment->user->profileUrl}\">{$comment->user->name}</a></cite></div>\n";
		$output_string .= "\t\t\t\t<div class=\"friendfeedcommenttext\">" . make_clickable($comment->body) . "</div><div style=\"clear:both;\"></div>\n";
		$output_string .= "\t\t\t</li>\n";	
		
		return $output_string;
	}
	
	//Get an entry from the list by it's entry id
	function get_discussion_by_id($entryid) {
		$ffdiscussions = get_option($this->discussions_key);
		
		foreach ($ffdiscussions as $discussion) {
			if ($discussion['id'] == $entryid) {
				return $discussion;
			}
		}
		
		return null;
	}
	
	//Render the output of the plugin
	function display_comments() {
		global $post;
		$ffdiscussions = get_option($this->discussions_key);
		$title = $this->format_title(the_title('','',false));
		$options = get_option($this->options_key);
		if ($this->is_debug()) {
			$this->printr($options);
			$this->printr($ffdiscussions);
			$this->printr($title);
		}
		
		if ($ffdiscussions && $ffdiscussions[$title]) {
			$discussion = $ffdiscussions[$title];
			$ffpostid = $discussion['id'];
?><script type="text/javascript">var friendFeedTasks<?=$post->ID?> = new friendFeedTasks('<?=$ffpostid?>');</script>
	<div id="friendfeedcomments_<?=$ffpostid?>" class="friendfeedcomments">
		<div id="friendfeedcommentslink_<?=$ffpostid?>" class="friendfeedcommentslink"><img src="<?=$this->plugin_path?>friendfeedicon.png" alt="" class="friendfeedicon" id="friendfeedicon_<?=$ffpostid?>" /><?php
			
			$comment_count = (is_array($discussion['comments']) ? count($discussion['comments']) : 0);
			$like_count = (is_array($discussion['likes']) ? count($discussion['likes']) : 0);
			
			if ($options['headline']) {
				$headline_text = str_replace('{comments}', $comment_count, $options['headline']);
				$headline_text = str_replace('{likes}', $like_count, $headline_text);
			} else {
				$headline_text = 'On FriendFeed, this post was ';
				$headline_text .= "liked by $like_count " . ($like_count <> 1 ? 'people' : 'person');
				$headline_text .= " and ";
				$headline_text .= " commented on $comment_count" . ($comment_count <> 1 ? ' times' : ' time');						
			}
			
			print $headline_text;
?> 	<a href="" class="togglefriendfeedcommentslink" id="ff_togglecommentslink_<?=$ffpostid?>" onclick="return false;" onfocus="blur();"><?=($options['showcomments'] ? 'hide' : 'show')?></a></div>
		<div id="friendfeeddiscussions_<?=$ffpostid?>" class="friendfeeddiscussions" style="display: <?=($options['showcomments'] ? 'block' : 'none')?>">
			<div class="friendfeedpermalink"><a href="http://www.friendfeed.com/e/<?=$ffpostid?>">View this post on FriendFeed</a></div>
<?php
			print "\t\t<div id=\"friendfeedlikeslist_$ffpostid\" class=\"friendfeedlikeslist\">";
			if ($discussion['likes']) {
				print $this->render_likes_list($discussion['likes']);
			}
			print "</div>\n";
			
			print "\t\t<div id=\"friendfeedcommentslist_$ffpostid\" class=\"friendfeedcommentslist\">\n\t\t";
			if ($discussion['comments']) {
				print $this->render_comments_list($discussion['comments']);
			}	
			print "</div>\n";
			
			if ($options['showform']) {
					$username = '';
					$apikey = '';
					$auth = $this->get_authentication();
					
					if ($auth) {
						$username = $auth[0];
						$apikey = $auth[1];
					}
?>
			<form class="ff_commentsform" id="ff_commentsform_<?=$ffpostid?>" method="post" action="">
			<h3>Add a comment on FriendFeed</h3>
			<textarea class="ff_newcomment" id="ff_newcomment_<?=$ffpostid?>" name="ff_newcomment"></textarea><br />
			<div class="ff_authentry" id="ff_authentry_<?=$ffpostid?>" style="display:<?=($username ? 'none' : 'block')?>;">
			<label for="ff_username_<?=$ffpostid?>">Username:</label><input type="text" name="ff_username" id="ff_username_<?=$ffpostid?>" value="<?=$username?>" /><br />
			<label for="ff_apikey_<?=$ffpostid?>">API Key<sup><a href="https://friendfeed.com/account/api" title="Get your API key from here" target="_BLANK">?</a></sup>:</label><input type="password" name="ff_apikey" id="ff_apikey_<?=$ffpostid?>" value="<?=$apikey?>" /><br />
			<label for="ff_remember_<?=$ffpostid?>">Remember:</label><input type="checkbox" name="ff_remember" id="ff_remember_<?=$ffpostid?>" />
			</div>
			<div style="display:<?=($username ? 'block' : 'none')?>;" class="ff_authsaved" id="ff_authsaved_<?=$ffpostid?>">
			Logged in as <span id="ff_usernamedisplay_<?=$ffpostid?>"><?=$username?></span> [<a href="" id="ff_forgetauth_<?=$ffpostid?>">logout</a>]
			</div>
			<input class="ff_submitcomment" id="ff_submitcomment_<?=$ffpostid?>" name="ff_submitcomment" value="Add comment" type="submit" />
			<input type="button" id="ff_likeentry_<?=$ffpostid?>" name="ff_likeentry" value="Like this post" style="<?=($options['nickname'] == $username ? 'display:none;' : '')?>" />
			<img src="<?=$this->plugin_path?>loading.gif" alt="" id="ff_loadingimage_<?=$ffpostid?>" style="display:none;" />
			<input type="hidden" name="ff_entryid" value="<?=$ffpostid?>" /> 
			</form>
			
			<script type="text/javascript">
			$('ff_commentsform_<?=$ffpostid?>').observe('submit', friendFeedTasks<?=$post->ID?>.addComment.bindAsEventListener(friendFeedTasks<?=$post->ID?>));
			$('ff_likeentry_<?=$ffpostid?>').observe('click', friendFeedTasks<?=$post->ID?>.addLike.bindAsEventListener(friendFeedTasks<?=$post->ID?>));
			$('ff_forgetauth_<?=$ffpostid?>').observe('click', friendFeedTasks<?=$post->ID?>.logout.bindAsEventListener(friendFeedTasks<?=$post->ID?>));
			</script>
<?php				
			}
?>		
			<script type="text/javascript">
			$('ff_togglecommentslink_<?=$ffpostid?>').observe('click', friendFeedTasks<?=$post->ID?>.toggleDiscussions.bindAsEventListener(friendFeedTasks<?=$post->ID?>));
			</script>
		</div>
	</div>
<?php
		}
	}
}

$FriendFeedPostComments = new FriendFeedPostComments();

function wp_ffcomments()
{
	global $FriendFeedPostComments;
	$FriendFeedPostComments->display_comments();
}
?>